#ifndef YERBACON_PARSER_HPP
#define YERBACON_PARSER_HPP

#include <string>
#include "ParseComponents.hpp"
#include "../Yerbacon.hpp"
#include <cstdarg>
#include <concepts>

namespace Parser {
    class ParsingException: public Yerbacon::Exception {
        using Yerbacon::Exception::Exception;
    };
    ParseTree parseVector(const vector<tok>& lexed) {
        ParseTree parseTree;
        using namespace StandardComponents;
        using enum tok::type;
        unsigned int i = 0;
        const auto nextAre = [&i, &lexed](convertible_to<unsigned int> auto nextValues,...) {
            va_list argumentsList;
            va_start(argumentsList, nextValues);
            for (decltype(nextValues) j = 1; j <= nextValues; ++j) {
                if (!cmp_greater(lexed.size() - i, nextValues) || lexed[i + j].toktype != va_arg(argumentsList, decltype(nextValues))) {
                    va_end(argumentsList);
                    return false;
                }
            }
            va_end(argumentsList);
            return true;
        };
        for (;i < lexed.size(); ++i) {
            const tok& current = lexed[i], next = (i != lexed.size() - 1) ? lexed[i + 1] : tok(UNEXPECTED, string(), current.line);

            switch (current.toktype) {
                case STRING: parseTree << types::String(current.toktext.data()); break;
                case IDENTIFIER: {
                    if (current.toktext == "class") {
                        if (next.toktype == IDENTIFIER) {
                            parseTree << Class(next.toktext); ++i;
                        } else throw ParsingException('"' + next.toktext + "\" is not a valid class identifier");
                    } else {
                        if ((lexed.size() - i) > 2) {
                            bool isFinalDefine = nextAre(2, TAG, DEFINE);
                            if (isFinalDefine || next.toktype == DEFINE) {
                                parseTree << Define(isFinalDefine, current.toktext);
                                i += isFinalDefine ? 2 : 1;
                            } else {
                                parseTree << Reference(current.toktext);
                            }
                        }
                    }
                }
                default: break;
            }
        }
        return parseTree;
    }
}

#endif //YERBACON_PARSER_HPP